//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File ioservice.hpp
// Date 11.03.08 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _IO_SERVICE_H_
#define _IO_SERVICE_H_

//***************************************************************************
// Include
//***************************************************************************

class IoService
{
   public:

      enum Misc
      {
         protocol = 0xA0               // random value ;)
      };

      enum Command
      {
         cNone = na,

         // from PC
         
         cGetTime        = 0x01,
         cDigitalOut     = 0x02,
         cDigitalOutBit  = 0x03,
         cAnalogOut      = 0x04,
         cGetInputs      = 0x05,
         cRecordGhostCar = 0x06,
         
         // to PC
         
         cDigitalIn      = 0x07,
         cAnalogIn       = 0x08,
         cBoardTime      = 0x09,
         cDebug          = 0x0A
      };

      enum GhostCarScale
      {
         // interrupt called every 1ms, this is the scale factor for
         // the ghost car recording intervall

         gcScale100 = 100,           // intervall 100ms
         gcScale200 = 200,           // intervall 200ms
         gcScale300 = 300,           // intervall 300ms
         gcScale400 = 400,           // intervall 400ms
         gcScale500 = 500,           // intervall 500ms
      };

#ifndef MKSKETCH
#  pragma pack(push, 1)
#endif

      // from PC

      struct DigitalOutput      // 4 + 2 byte
      {
          dword value;
      };

      struct DigitalOutputBit   // 2 + 2 byte
      {
          byte bit;
          byte state;
      };

      struct GhostCarOutput     // 2 + 2 byte
      {
          byte volt;
          byte ampere;
      };

      struct AnalogOutput       // 2 + 2 byte
      {
          byte bit;
          byte value;
      };

      struct RecordGhostCar     // 2 + 2 byte
      {
          byte cycle;
          char bits;            //  ampereIn << 4 | voltIn
      };

      // to PC

      struct DigitalInput       // 8 + 2 byte
      {
          dword time;
          dword value;
      };

      struct AnalogInput        // 2 + 2 byte
      {
          byte volt;
          byte ampere;
      };

      struct DebugValue         // 58 + 2 byte
      {
          char string[49+TB];
          dword value;
      };

#ifndef MKSKETCH
#  pragma pack(pop)
#endif

};

typedef IoService Ios;

//***************************************************************************
#endif //  _IO_SERVICE_H_
