//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File iothread.hpp
// Date 28.11.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _IO_THREAD_H_
#define _IO_THREAD_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QThread>

#include <common.hpp>
#include <iointerface.hpp>

//***************************************************************************
// I/O Thread
//***************************************************************************

class IoThread : public QThread, public SlotService, public IoService
{
      Q_OBJECT
      
   public:

      // object
      
      IoThread();
      virtual ~IoThread();

      // interface

      int init();
      int exit();
      int open();
      int close();
      void stop()                       { running = no; }
      void setTestMode(int aFlag)       { testMode = aFlag; }
      int writeBit(int bit, int value)  { return ioDevice->writeBit(bit, value); }
      int readOutBit(int bit)           { return ioDevice->readOutBit(bit); }
      int isOpen()                      { return ioDevice->isOpen(); }
      void changeDevice(const char* dev);
      int waitIo();
      int lookIo();
      void recordGhostCar(int vBit, int iBit)  { return ioDevice->recordGhostCar(vBit, iBit); }
      int getGcScale()                  { return ioDevice->getGcScale(); }
      void control();

      // get / set

      const char* getDevice()           { return device; }
      void setDevice(const char* dev);

   signals:

      void onDigitalInput(const DigitalEvent &ioEvent);
      void onAnalogInput(const AnalogEvent &ioEvent);

   protected:
      
      void run();
      int atTrouble();

      // data

      char device[100+TB];
      int testMode;
      IoInterface* ioDevice;
      int running;
      byte command;
};

//***************************************************************************
#endif // _IO_THREAD_H_
