//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File setup.hpp
// Date 05.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _SETUP_H_
#define _SETUP_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QDialog>
#include <QSettings>

#include <ui_setup.h>
#include <common.hpp>

//***************************************************************************
// Class LinslotWindow
//***************************************************************************

class SetupDialog : public QDialog, public Ui::SetupDialog, public SlotService
{
      Q_OBJECT
      
   public:
      
      // object
      
      SetupDialog(QString configPath);
      ~SetupDialog();

      // gettings

      QSettings* getSettings()      { return settings; }

      int getRoundCount()           { return roundCount; }
      int getRoundLimit()           { return roundLimit; }
      double getSlotLength()        { return slotLength; }  // Meter
      const char* getUsbDevice()    { return usbDevice; }
      int getMaxTimeRoundRace()     { return maxTimeRoundRace; }
      int getMaxTimeTraining()      { return maxTimeTraining; }
      int getSpeedFactor()          { return speedFactor; }
      const char* getCourseName()   { return courseName; }
      const char* getDatabaseName() { return databaseName; }
      const char* getResourcePath() { return resourcePath; }
      double getAverageRound()      { return averageRound; }
      int getAbortAtJumpTheGun()    { return abortAtJumpTheGun; }
      int getPenaltyAtJumpTheGun()  { return penaltyAtJumpTheGun; }

      double getFuelMax()           { return fuelMax; }
      double getFuelPerRound()      { return fuelPerRound; }
      int getFuelingActive()        { return fuelingActive; }
      int getFuelingDelay()         { return fuelingDelay; }
      int getFuelingPerSecond()     { return fuelingPerSecond; }
      int getFuelFactorFastLap()    { return fuelFactorFastLap; }
      int getFuelFactorSlowLap()    { return fuelFactorSlowLap; }
      int getFuelPenaltyTime()      { return fuelPenaltyTime; }

      int getGhostcarInvert()       { return yes; }  // todo: einstellbar

   protected:
      
      QSettings* settings;
      int roundCount;              // round limit for races
      int roundLimit;              // round limit for training
      double slotLength;
      int maxTimeRoundRace;
      int maxTimeTraining;
      int speedFactor;
      char courseName[100+TB];
      char usbDevice[100+TB];
      char databaseName[100+TB];
      char resourcePath[1000+TB];
      int abortAtJumpTheGun;
      int penaltyAtJumpTheGun;
      double fuelPerRound;
      double fuelMax;
      int fuelingDelay;
      int fuelingPerSecond;
      int fuelFactorFastLap;
      int fuelFactorSlowLap; 
      double averageRound;
      int fuelingActive;
      int fuelPenaltyTime;

   private slots:

      void on_tabWidget_currentChanged(int tab);
      void on_pushButtonAddDriver_clicked();
      void on_pushButtonDelDriver_clicked();
      void on_spinBoxRoundCount_valueChanged(int value);
      void on_spinBoxTrainingRoundLimit_valueChanged(int value);
      void on_spinBoxJumpTheGunPenaltyTime_valueChanged(int value);
      void on_doubleSpinBoxSlotLength_valueChanged(double value);
      void on_comboBoxDevice_currentIndexChanged(const QString text);
      void on_comboBoxDevice_editTextChanged(const QString text);
      void on_lineEditCourse_editingFinished();
      void on_lineEditDbName_editingFinished();
      void on_lineEditResourcePath_editingFinished();
      void on_timeEditRoundRace_editingFinished();
      void on_timeEditTraining_editingFinished();
      void on_radioButtonFaktor1_toggled(bool checked);
      void on_radioButtonFaktor24_toggled(bool checked);
      void on_radioButtonFaktor32_toggled(bool checked);
      void on_radioButtonFaktor43_toggled(bool checked);
      void on_radioButtonJumpTheGunAbort_toggled(bool checked);
      void on_radioButtonJumpTheGunPenalty_toggled(bool checked);
      void on_doubleSpinBoxFuelMax_valueChanged(double value);
      void on_doubleSpinBoxFuelPerRound_valueChanged(double value);
      void on_spinBoxFuelingDelay_valueChanged(int value);
      void on_spinBoxFuelingPerSecond_valueChanged(int value);
      void on_spinBoxFuelFactorFastLap_valueChanged(int value);
      void on_spinBoxFuelFactorSlowLap_valueChanged(int value);
      void on_doubleSpinBoxAverageRound_valueChanged(double value);
      void on_checkBoxFueling_stateChanged(int state);
      void on_spinBoxFuelPenaltyTime_valueChanged(int value);
      void outputSignalChanged(int row, int col);
      void inputSignalChanged(int row, int col);

};

//***************************************************************************
#endif // _SETUP_H_
